;;; helm-aliases.el --- Helm aliases for helm obsoletes functions. -*- lexical-binding: t -*-

;; Copyright (C) 2012 ~ 2014 Thierry Volpiatto <thierry.volpiatto@gmail.com>

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Code:

(require 'cl-lib)


;;; Helper functions to create aliases with old helm definitions
;;  prefixed with "helm-c-"
;;
(defun helm-alias-p (sym)
  (cond ((boundp sym)
         (not (eq (indirect-variable sym) sym)))
        ((fboundp sym)
         (symbolp (symbol-function sym)))
        (t nil)))

(defun helm-check-conflicting-prefixes ()
  (cl-loop for s in (all-completions "helm-c-" obarray)
           for rep = (replace-regexp-in-string "helm-c-" "helm-" s)
           when (or (and (not (helm-alias-p (intern s))) (fboundp (intern rep)))
                    (and (not (helm-alias-p (intern s))) (boundp (intern rep))))
           collect rep))

(defun helm-collect-functions-with-bad-prefix ()
  (cl-loop for s in (all-completions "helm-c-" obarray)
           for sym = (intern s)
           when (and (not (helm-alias-p sym)) (fboundp sym))
           collect s))

(defun helm-collect-vars-with-bad-prefix ()
  (cl-loop for s in (all-completions "helm-c-" obarray)
           for sym = (intern s)
           when (and (not (helm-alias-p sym)) (boundp sym))
           collect s))

(defun helm-insert-fn-aliases ()
  (cl-loop for s in (helm-collect-functions-with-bad-prefix)
           for rep = (replace-regexp-in-string "helm-c-" "helm-" s)
           do (insert (format "(defalias '%s '%s)\n(make-obsolete '%s '%s \"1.5.1\")\n" s rep s rep))))

(defun helm-insert-var-aliases ()
  (cl-loop for s in (helm-collect-vars-with-bad-prefix)
           for rep = (replace-regexp-in-string "helm-c-" "helm-" s)
           do (insert (format "(defvaralias '%s '%s)\n(make-obsolete-variable '%s '%s \"1.5.1\")\n" s rep s rep))))


;;; Alias old functions prefixed with "helm-c-"
;;
;;
(defalias 'helm-c-etags-default-action 'helm-etags-default-action)
(make-obsolete 'helm-c-etags-default-action 'helm-etags-default-action "1.5.1")
(defalias 'helm-c-simple-call-tree-candidates 'helm-simple-call-tree-candidates)
(make-obsolete 'helm-c-simple-call-tree-candidates 'helm-simple-call-tree-candidates "1.5.1")
(defalias 'helm-c-show-info-in-mode-line 'helm-show-info-in-mode-line)
(make-obsolete 'helm-c-show-info-in-mode-line 'helm-show-info-in-mode-line "1.5.1")
(defalias 'helm-c-gentoo-eshell-action 'helm-gentoo-eshell-action)
(make-obsolete 'helm-c-gentoo-eshell-action 'helm-gentoo-eshell-action "1.5.1")
(defalias 'helm-c-grep-highlight-match 'helm-grep-highlight-match)
(make-obsolete 'helm-c-grep-highlight-match 'helm-grep-highlight-match "1.5.1")
(defalias 'helm-c-apropos 'helm-apropos)
(make-obsolete 'helm-c-apropos 'helm-apropos "1.5.1")
(defalias 'helm-c-grep-guess-extensions 'helm-grep-guess-extensions)
(make-obsolete 'helm-c-grep-guess-extensions 'helm-grep-guess-extensions "1.5.1")
(defalias 'helm-c-highlight-w3m-bookmarks 'helm-highlight-w3m-bookmarks)
(make-obsolete 'helm-c-highlight-w3m-bookmarks 'helm-highlight-w3m-bookmarks "1.5.1")
(defalias 'helm-c-emms-files-modifier 'helm-emms-files-modifier)
(make-obsolete 'helm-c-emms-files-modifier 'helm-emms-files-modifier "1.5.1")
(defalias 'helm-c-apt-reinstall 'helm-apt-reinstall)
(make-obsolete 'helm-c-apt-reinstall 'helm-apt-reinstall "1.5.1")
(defalias 'helm-c-bbdb-compose-mail 'helm-bbdb-compose-mail)
(make-obsolete 'helm-c-bbdb-compose-mail 'helm-bbdb-compose-mail "1.5.1")
(defalias 'helm-c-gentoo-init-list 'helm-gentoo-init-list)
(make-obsolete 'helm-c-gentoo-init-list 'helm-gentoo-init-list "1.5.1")
(defalias 'helm-c-gentoo-get-world 'helm-gentoo-get-world)
(make-obsolete 'helm-c-gentoo-get-world 'helm-gentoo-get-world "1.5.1")
(defalias 'helm-c-reset-adaptative-history 'helm-reset-adaptative-history)
(make-obsolete 'helm-c-reset-adaptative-history 'helm-reset-adaptative-history "1.5.1")
(defalias 'helm-c-highlight-bookmark 'helm-highlight-bookmark)
(make-obsolete 'helm-c-highlight-bookmark 'helm-highlight-bookmark "1.5.1")
(defalias 'helm-c-locate-init 'helm-locate-init)
(make-obsolete 'helm-c-locate-init 'helm-locate-init "1.5.1")
(defalias 'helm-c-delete-file 'helm-delete-file)
(make-obsolete 'helm-c-delete-file 'helm-delete-file "1.5.1")
(defalias 'helm-c-regexp-persistent-action 'helm-regexp-persistent-action)
(make-obsolete 'helm-c-regexp-persistent-action 'helm-regexp-persistent-action "1.5.1")
(defalias 'helm-c-grep-split-line 'helm-grep-split-line)
(make-obsolete 'helm-c-grep-split-line 'helm-grep-split-line "1.5.1")
(defalias 'helm-c-filtered-candidate-transformer-file-line-1 'helm-filtered-candidate-transformer-file-line-1)
(make-obsolete 'helm-c-filtered-candidate-transformer-file-line-1 'helm-filtered-candidate-transformer-file-line-1 "1.5.1")
(defalias 'helm-c-info-display-to-real 'helm-info-display-to-real)
(make-obsolete 'helm-c-info-display-to-real 'helm-info-display-to-real "1.5.1")
(defalias 'helm-c-symbolify 'helm-symbolify)
(make-obsolete 'helm-c-symbolify 'helm-symbolify "1.5.1")
(defalias 'helm-c-zgrep-buffers 'helm-zgrep-buffers)
(make-obsolete 'helm-c-zgrep-buffers 'helm-zgrep-buffers "1.5.1")
(defalias 'helm-c-apt-persistent-action 'helm-apt-persistent-action)
(make-obsolete 'helm-c-apt-persistent-action 'helm-apt-persistent-action "1.5.1")
(defalias 'helm-c-pdfgrep-init 'helm-pdfgrep-init)
(make-obsolete 'helm-c-pdfgrep-init 'helm-pdfgrep-init "1.5.1")
(defalias 'helm-c-walk-directory 'helm-walk-directory)
(make-obsolete 'helm-c-walk-directory 'helm-walk-directory "1.5.1")
(defalias 'helm-c-grep-init 'helm-grep-init)
(make-obsolete 'helm-c-grep-init 'helm-grep-init "1.5.1")
(defalias 'helm-c-ucs-forward-char 'helm-ucs-forward-char)
(make-obsolete 'helm-c-ucs-forward-char 'helm-ucs-forward-char "1.5.1")
(defalias 'helm-c-complete-file-name-at-point 'helm-complete-file-name-at-point)
(make-obsolete 'helm-c-complete-file-name-at-point 'helm-complete-file-name-at-point "1.5.1")
(defalias 'helm-c-timer-real-to-display 'helm-timer-real-to-display)
(make-obsolete 'helm-c-timer-real-to-display 'helm-timer-real-to-display "1.5.1")
(defalias 'helm-c-ucs-init 'helm-ucs-init)
(make-obsolete 'helm-c-ucs-init 'helm-ucs-init "1.5.1")
(defalias 'helm-c-bookmark-run-delete 'helm-bookmark-run-delete)
(make-obsolete 'helm-c-bookmark-run-delete 'helm-bookmark-run-delete "1.5.1")
(defalias 'helm-c-uniq-list 'helm-uniq-list)
(make-obsolete 'helm-c-uniq-list 'helm-uniq-list "1.5.1")
(defalias 'helm-c-pp-bookmarks 'helm-pp-bookmarks)
(make-obsolete 'helm-c-pp-bookmarks 'helm-pp-bookmarks "1.5.1")
(defalias 'helm-c-gentoo-get-url 'helm-gentoo-get-url)
(make-obsolete 'helm-c-gentoo-get-url 'helm-gentoo-get-url "1.5.1")
(defalias 'helm-c-top-sh-persistent-action 'helm-top-sh-persistent-action)
(make-obsolete 'helm-c-top-sh-persistent-action 'helm-top-sh-persistent-action "1.5.1")
(defalias 'helm-c-gentoo-get-use 'helm-gentoo-get-use)
(make-obsolete 'helm-c-gentoo-get-use 'helm-gentoo-get-use "1.5.1")
(defalias 'helm-c-grep-action 'helm-grep-action)
(make-obsolete 'helm-c-grep-action 'helm-grep-action "1.5.1")
(defalias 'helm-c-file-buffers 'helm-file-buffers)
(make-obsolete 'helm-c-file-buffers 'helm-file-buffers "1.5.1")
(defalias 'helm-c-google-suggest-fetch 'helm-google-suggest-fetch)
(make-obsolete 'helm-c-google-suggest-fetch 'helm-google-suggest-fetch "1.5.1")
(defalias 'helm-c-arrange-type-attribute 'helm-arrange-type-attribute)
(make-obsolete 'helm-c-arrange-type-attribute 'helm-arrange-type-attribute "1.5.1")
(defalias 'helm-c-bookmark-w3m-setup-alist 'helm-bookmark-w3m-setup-alist)
(make-obsolete 'helm-c-bookmark-w3m-setup-alist 'helm-bookmark-w3m-setup-alist "1.5.1")
(defalias 'helm-c-yahoo-suggest-action 'helm-yahoo-suggest-action)
(make-obsolete 'helm-c-yahoo-suggest-action 'helm-yahoo-suggest-action "1.5.1")
(defalias 'helm-c-buffer-query-replace-1 'helm-buffer-query-replace-1)
(make-obsolete 'helm-c-buffer-query-replace-1 'helm-buffer-query-replace-1 "1.5.1")
(defalias 'helm-c-make-info-source 'helm-make-info-source)
(make-obsolete 'helm-c-make-info-source 'helm-make-info-source "1.5.1")
(defalias 'helm-c-open-dired 'helm-open-dired)
(make-obsolete 'helm-c-open-dired 'helm-open-dired "1.5.1")
(defalias 'helm-c-w3m-bookmarks-get-value 'helm-w3m-bookmarks-get-value)
(make-obsolete 'helm-c-w3m-bookmarks-get-value 'helm-w3m-bookmarks-get-value "1.5.1")
(defalias 'helm-c-kill-ring-transformer 'helm-kill-ring-transformer)
(make-obsolete 'helm-c-kill-ring-transformer 'helm-kill-ring-transformer "1.5.1")
(defalias 'helm-c-etags-init 'helm-etags-init)
(make-obsolete 'helm-c-etags-init 'helm-etags-init "1.5.1")
(defalias 'helm-c-w3m-rename-bookmark 'helm-w3m-rename-bookmark)
(make-obsolete 'helm-c-w3m-rename-bookmark 'helm-w3m-rename-bookmark "1.5.1")
(defalias 'helm-c-advice-update-current-display-string 'helm-advice-update-current-display-string)
(make-obsolete 'helm-c-advice-update-current-display-string 'helm-advice-update-current-display-string "1.5.1")
(defalias 'helm-c-regexp-get-line 'helm-regexp-get-line)
(make-obsolete 'helm-c-regexp-get-line 'helm-regexp-get-line "1.5.1")
(defalias 'helm-c-grep-run-other-window-action 'helm-grep-run-other-window-action)
(make-obsolete 'helm-c-grep-run-other-window-action 'helm-grep-run-other-window-action "1.5.1")
(defalias 'helm-c-regexp-kill-new 'helm-regexp-kill-new)
(make-obsolete 'helm-c-regexp-kill-new 'helm-regexp-kill-new "1.5.1")
(defalias 'helm-c-goto-line-with-adjustment 'helm-goto-line-with-adjustment)
(make-obsolete 'helm-c-goto-line-with-adjustment 'helm-goto-line-with-adjustment "1.5.1")
(defalias 'helm-c-grep-run-default-action 'helm-grep-run-default-action)
(make-obsolete 'helm-c-grep-run-default-action 'helm-grep-run-default-action "1.5.1")
(defalias 'helm-c-define-info-index-sources 'helm-define-info-index-sources)
(make-obsolete 'helm-c-define-info-index-sources 'helm-define-info-index-sources "1.5.1")
(defalias 'helm-c-transform-file-browse-url 'helm-transform-file-browse-url)
(make-obsolete 'helm-c-transform-file-browse-url 'helm-transform-file-browse-url "1.5.1")
(defalias 'helm-c-shell-command-if-needed 'helm-shell-command-if-needed)
(make-obsolete 'helm-c-shell-command-if-needed 'helm-shell-command-if-needed "1.5.1")
(defalias 'helm-c-kill-regexp-as-sexp 'helm-kill-regexp-as-sexp)
(make-obsolete 'helm-c-kill-regexp-as-sexp 'helm-kill-regexp-as-sexp "1.5.1")
(defalias 'helm-c-goto-next-or-prec-file 'helm-goto-next-or-prec-file)
(make-obsolete 'helm-c-goto-next-or-prec-file 'helm-goto-next-or-prec-file "1.5.1")
(defalias 'helm-c-simple-call-tree-functions-callers-init 'helm-simple-call-tree-functions-callers-init)
(make-obsolete 'helm-c-simple-call-tree-functions-callers-init 'helm-simple-call-tree-functions-callers-init "1.5.1")
(defalias 'helm-c-insert-file-name-completion-at-point 'helm-insert-file-name-completion-at-point)
(make-obsolete 'helm-c-insert-file-name-completion-at-point 'helm-insert-file-name-completion-at-point "1.5.1")
(defalias 'helm-c-point-file-in-dired 'helm-point-file-in-dired)
(make-obsolete 'helm-c-point-file-in-dired 'helm-point-file-in-dired "1.5.1")
(defalias 'helm-c-action-file-line-goto 'helm-action-file-line-goto)
(make-obsolete 'helm-c-action-file-line-goto 'helm-action-file-line-goto "1.5.1")
(defalias 'helm-c-info-goto 'helm-info-goto)
(make-obsolete 'helm-c-info-goto 'helm-info-goto "1.5.1")
(defalias 'helm-c-apt-install 'helm-apt-install)
(make-obsolete 'helm-c-apt-install 'helm-apt-install "1.5.1")
(defalias 'helm-c-skip-boring-buffers 'helm-skip-boring-buffers)
(make-obsolete 'helm-c-skip-boring-buffers 'helm-skip-boring-buffers "1.5.1")
(defalias 'helm-c-register-candidates 'helm-register-candidates)
(make-obsolete 'helm-c-register-candidates 'helm-register-candidates "1.5.1")
(defalias 'helm-c-find-function 'helm-find-function)
(make-obsolete 'helm-c-find-function 'helm-find-function "1.5.1")
(defalias 'helm-c-simple-call-tree-find-definition 'helm-simple-call-tree-find-definition)
(make-obsolete 'helm-c-simple-call-tree-find-definition 'helm-simple-call-tree-find-definition "1.5.1")
(defalias 'helm-c-apt-display-to-real 'helm-apt-display-to-real)
(make-obsolete 'helm-c-apt-display-to-real 'helm-apt-display-to-real "1.5.1")
(defalias 'helm-c-yaoddmuse-action-transformer 'helm-yaoddmuse-action-transformer)
(make-obsolete 'helm-c-yaoddmuse-action-transformer 'helm-yaoddmuse-action-transformer "1.5.1")
(defalias 'helm-c-org-keywords-init 'helm-org-keywords-init)
(make-obsolete 'helm-c-org-keywords-init 'helm-org-keywords-init "1.5.1")
(defalias 'helm-c-xrandr-screen 'helm-xrandr-screen)
(make-obsolete 'helm-c-xrandr-screen 'helm-xrandr-screen "1.5.1")
(defalias 'helm-c-simple-call-tree-persistent-action 'helm-simple-call-tree-persistent-action)
(make-obsolete 'helm-c-simple-call-tree-persistent-action 'helm-simple-call-tree-persistent-action "1.5.1")
(defalias 'helm-c-position 'helm-position)
(make-obsolete 'helm-c-position 'helm-position "1.5.1")
(defalias 'helm-c-imenu-candidates 'helm-imenu-candidates)
(make-obsolete 'helm-c-imenu-candidates 'helm-imenu-candidates "1.5.1")
(defalias 'helm-c-bbdb-candidates 'helm-bbdb-candidates)
(make-obsolete 'helm-c-bbdb-candidates 'helm-bbdb-candidates "1.5.1")
(defalias 'helm-c-ucs-persistent-forward 'helm-ucs-persistent-forward)
(make-obsolete 'helm-c-ucs-persistent-forward 'helm-ucs-persistent-forward "1.5.1")
(defalias 'helm-c-basename 'helm-basename)
(make-obsolete 'helm-c-basename 'helm-basename "1.5.1")
(defalias 'helm-c-files-in-all-dired-candidates 'helm-files-in-all-dired-candidates)
(make-obsolete 'helm-c-files-in-all-dired-candidates 'helm-files-in-all-dired-candidates "1.5.1")
(defalias 'helm-c-action-line-goto 'helm-action-line-goto)
(make-obsolete 'helm-c-action-line-goto 'helm-action-line-goto "1.5.1")
(defalias 'helm-c-grep-other-window 'helm-grep-other-window)
(make-obsolete 'helm-c-grep-other-window 'helm-grep-other-window "1.5.1")
(defalias 'helm-c-yahoo-suggest-fetch 'helm-yahoo-suggest-fetch)
(make-obsolete 'helm-c-yahoo-suggest-fetch 'helm-yahoo-suggest-fetch "1.5.1")
(defalias 'helm-c-external-commands-list-1 'helm-external-commands-list-1)
(make-obsolete 'helm-c-external-commands-list-1 'helm-external-commands-list-1 "1.5.1")
(defalias 'helm-c-buffers-list-persistent-action 'helm-buffers-list-persistent-action)
(make-obsolete 'helm-c-buffers-list-persistent-action 'helm-buffers-list-persistent-action "1.5.1")
(defalias 'helm-c-show-completion-init-overlay 'helm-show-completion-init-overlay)
(make-obsolete 'helm-c-show-completion-init-overlay 'helm-show-completion-init-overlay "1.5.1")
(defalias 'helm-c-shadow-boring-files 'helm-shadow-boring-files)
(make-obsolete 'helm-c-shadow-boring-files 'helm-shadow-boring-files "1.5.1")
(defalias 'helm-c-bookmark-local-files-setup-alist 'helm-bookmark-local-files-setup-alist)
(make-obsolete 'helm-c-bookmark-local-files-setup-alist 'helm-bookmark-local-files-setup-alist "1.5.1")
(defalias 'helm-c-highlight-local-use 'helm-highlight-local-use)
(make-obsolete 'helm-c-highlight-local-use 'helm-highlight-local-use "1.5.1")
(defalias 'helm-c-w3m-delete-bookmark 'helm-w3m-delete-bookmark)
(make-obsolete 'helm-c-w3m-delete-bookmark 'helm-w3m-delete-bookmark "1.5.1")
(defalias 'helm-c-skip-current-file 'helm-skip-current-file)
(make-obsolete 'helm-c-skip-current-file 'helm-skip-current-file "1.5.1")
(defalias 'helm-c-goto-next-file 'helm-goto-next-file)
(make-obsolete 'helm-c-goto-next-file 'helm-goto-next-file "1.5.1")
(defalias 'helm-c-info-init 'helm-info-init)
(make-obsolete 'helm-c-info-init 'helm-info-init "1.5.1")
(defalias 'helm-c-query-replace-args 'helm-query-replace-args)
(make-obsolete 'helm-c-query-replace-args 'helm-query-replace-args "1.5.1")
(defalias 'helm-c-bookmark-info-setup-alist 'helm-bookmark-info-setup-alist)
(make-obsolete 'helm-c-bookmark-info-setup-alist 'helm-bookmark-info-setup-alist "1.5.1")
(defalias 'helm-c-ratpoison-commands-execute 'helm-ratpoison-commands-execute)
(make-obsolete 'helm-c-ratpoison-commands-execute 'helm-ratpoison-commands-execute "1.5.1")
(defalias 'helm-c-ucs-persistent-backward 'helm-ucs-persistent-backward)
(make-obsolete 'helm-c-ucs-persistent-backward 'helm-ucs-persistent-backward "1.5.1")
(defalias 'helm-c-info-pages-init 'helm-info-pages-init)
(make-obsolete 'helm-c-info-pages-init 'helm-info-pages-init "1.5.1")
(defalias 'helm-c-grep-save-results 'helm-grep-save-results)
(make-obsolete 'helm-c-grep-save-results 'helm-grep-save-results "1.5.1")
(defalias 'helm-c-simple-call-tree-callers-functions-init 'helm-simple-call-tree-callers-functions-init)
(make-obsolete 'helm-c-simple-call-tree-callers-functions-init 'helm-simple-call-tree-callers-functions-init "1.5.1")
(defalias 'helm-c-read-file-name 'helm-read-file-name)
(make-obsolete 'helm-c-read-file-name 'helm-read-file-name "1.5.1")
(defalias 'helm-c-org-keywords-insert 'helm-org-keywords-insert)
(make-obsolete 'helm-c-org-keywords-insert 'helm-org-keywords-insert "1.5.1")
(defalias 'helm-c-call-interactively 'helm-call-interactively)
(make-obsolete 'helm-c-call-interactively 'helm-call-interactively "1.5.1")
(defalias 'helm-c-adaptive-compare 'helm-adaptive-compare)
(make-obsolete 'helm-c-adaptive-compare 'helm-adaptive-compare "1.5.1")
(defalias 'helm-c-filtered-candidate-transformer-file-line 'helm-filtered-candidate-transformer-file-line)
(make-obsolete 'helm-c-filtered-candidate-transformer-file-line 'helm-filtered-candidate-transformer-file-line "1.5.1")
(defalias 'helm-c-buffer-list 'helm-buffer-list)
(make-obsolete 'helm-c-buffer-list 'helm-buffer-list "1.5.1")
(defalias 'helm-c-gentoo-get-local-use 'helm-gentoo-get-local-use)
(make-obsolete 'helm-c-gentoo-get-local-use 'helm-gentoo-get-local-use "1.5.1")
(defalias 'helm-c-thing-before-point 'helm-thing-before-point)
(make-obsolete 'helm-c-thing-before-point 'helm-thing-before-point "1.5.1")
(defalias 'helm-c-ratpoison-commands-display-to-real 'helm-ratpoison-commands-display-to-real)
(make-obsolete 'helm-c-ratpoison-commands-display-to-real 'helm-ratpoison-commands-display-to-real "1.5.1")
(defalias 'helm-c-imenu-default-action 'helm-imenu-default-action)
(make-obsolete 'helm-c-imenu-default-action 'helm-imenu-default-action "1.5.1")
(defalias 'helm-c-describe-function 'helm-describe-function)
(make-obsolete 'helm-c-describe-function 'helm-describe-function "1.5.1")
(defalias 'helm-c-colors-get-name 'helm-colors-get-name)
(make-obsolete 'helm-c-colors-get-name 'helm-colors-get-name "1.5.1")
(defalias 'helm-c-bookmark-help 'helm-bookmark-help)
(make-obsolete 'helm-c-bookmark-help 'helm-bookmark-help "1.5.1")
(defalias 'helm-c-ucs-persistent-delete 'helm-ucs-persistent-delete)
(make-obsolete 'helm-c-ucs-persistent-delete 'helm-ucs-persistent-delete "1.5.1")
(defalias 'helm-c-set-variable 'helm-set-variable)
(make-obsolete 'helm-c-set-variable 'helm-set-variable "1.5.1")
(defalias 'helm-c-w3m-browse-bookmark 'helm-w3m-browse-bookmark)
(make-obsolete 'helm-c-w3m-browse-bookmark 'helm-w3m-browse-bookmark "1.5.1")
(defalias 'helm-c-grep-save-results-1 'helm-grep-save-results-1)
(make-obsolete 'helm-c-grep-save-results-1 'helm-grep-save-results-1 "1.5.1")
(defalias 'helm-c-adaptative-maybe-load-history 'helm-adaptative-maybe-load-history)
(make-obsolete 'helm-c-adaptative-maybe-load-history 'helm-adaptative-maybe-load-history "1.5.1")
(defalias 'helm-c-w32-pathname-transformer 'helm-w32-pathname-transformer)
(make-obsolete 'helm-c-w32-pathname-transformer 'helm-w32-pathname-transformer "1.5.1")
(defalias 'helm-c-top-display-to-real 'helm-top-display-to-real)
(make-obsolete 'helm-c-top-display-to-real 'helm-top-display-to-real "1.5.1")
(defalias 'helm-c-open-file-externally 'helm-open-file-externally)
(make-obsolete 'helm-c-open-file-externally 'helm-open-file-externally "1.5.1")
(defalias 'helm-c-colors-get-rgb 'helm-colors-get-rgb)
(make-obsolete 'helm-c-colors-get-rgb 'helm-colors-get-rgb "1.5.1")
(defalias 'helm-c-org-headline-refile 'helm-org-headline-refile)
(make-obsolete 'helm-c-org-headline-refile 'helm-org-headline-refile "1.5.1")
(defalias 'helm-c-stumpwm-commands-init 'helm-stumpwm-commands-init)
(make-obsolete 'helm-c-stumpwm-commands-init 'helm-stumpwm-commands-init "1.5.1")
(defalias 'helm-c-highlight-world 'helm-highlight-world)
(make-obsolete 'helm-c-highlight-world 'helm-highlight-world "1.5.1")
(defalias 'helm-c-top-sh 'helm-top-sh)
(make-obsolete 'helm-c-top-sh 'helm-top-sh "1.5.1")
(defalias 'helm-c-bbdb-get-record 'helm-bbdb-get-record)
(make-obsolete 'helm-c-bbdb-get-record 'helm-bbdb-get-record "1.5.1")
(defalias 'helm-c-persistent-xfont-action 'helm-persistent-xfont-action)
(make-obsolete 'helm-c-persistent-xfont-action 'helm-persistent-xfont-action "1.5.1")
(defalias 'helm-c-latex-math-candidates 'helm-latex-math-candidates)
(make-obsolete 'helm-c-latex-math-candidates 'helm-latex-math-candidates "1.5.1")
(defalias 'helm-c-ucs-backward-char 'helm-ucs-backward-char)
(make-obsolete 'helm-c-ucs-backward-char 'helm-ucs-backward-char "1.5.1")
(defalias 'helm-c-calculate-ucs-max-len 'helm-calculate-ucs-max-len)
(make-obsolete 'helm-c-calculate-ucs-max-len 'helm-calculate-ucs-max-len "1.5.1")
(defalias 'helm-c-etags-mtime 'helm-etags-mtime)
(make-obsolete 'helm-c-etags-mtime 'helm-etags-mtime "1.5.1")
(defalias 'helm-c-google-suggest-action 'helm-google-suggest-action)
(make-obsolete 'helm-c-google-suggest-action 'helm-google-suggest-action "1.5.1")
(defalias 'helm-c-org-headline-insert-link-to-headline 'helm-org-headline-insert-link-to-headline)
(make-obsolete 'helm-c-org-headline-insert-link-to-headline 'helm-org-headline-insert-link-to-headline "1.5.1")
(defalias 'helm-c-bbdb-create-contact 'helm-bbdb-create-contact)
(make-obsolete 'helm-c-bbdb-create-contact 'helm-bbdb-create-contact "1.5.1")
(defalias 'helm-c-elisp-library-scan-init 'helm-elisp-library-scan-init)
(make-obsolete 'helm-c-elisp-library-scan-init 'helm-elisp-library-scan-init "1.5.1")
(defalias 'helm-c-apt-refresh 'helm-apt-refresh)
(make-obsolete 'helm-c-apt-refresh 'helm-apt-refresh "1.5.1")
(defalias 'helm-c-kill-regexp 'helm-kill-regexp)
(make-obsolete 'helm-c-kill-regexp 'helm-kill-regexp "1.5.1")
(defalias 'helm-c-gentoo-default-action 'helm-gentoo-default-action)
(make-obsolete 'helm-c-gentoo-default-action 'helm-gentoo-default-action "1.5.1")
(defalias 'helm-c-grep-buffers-1 'helm-grep-buffers-1)
(make-obsolete 'helm-c-grep-buffers-1 'helm-grep-buffers-1 "1.5.1")
(defalias 'helm-c-adaptive-store-selection 'helm-adaptive-store-selection)
(make-obsolete 'helm-c-adaptive-store-selection 'helm-adaptive-store-selection "1.5.1")
(defalias 'helm-c-browse-url 'helm-browse-url)
(make-obsolete 'helm-c-browse-url 'helm-browse-url "1.5.1")
(defalias 'helm-c-google-suggest-emacs-lisp 'helm-google-suggest-emacs-lisp)
(make-obsolete 'helm-c-google-suggest-emacs-lisp 'helm-google-suggest-emacs-lisp "1.5.1")
(defalias 'helm-c-ucs-persistent-insert 'helm-ucs-persistent-insert)
(make-obsolete 'helm-c-ucs-persistent-insert 'helm-ucs-persistent-insert "1.5.1")
(defalias 'helm-c-xrandr-output 'helm-xrandr-output)
(make-obsolete 'helm-c-xrandr-output 'helm-xrandr-output "1.5.1")
(defalias 'helm-c-kill-ring-action 'helm-kill-ring-action)
(make-obsolete 'helm-c-kill-ring-action 'helm-kill-ring-action "1.5.1")
(defalias 'helm-c-org-keywords-candidates 'helm-org-keywords-candidates)
(make-obsolete 'helm-c-org-keywords-candidates 'helm-org-keywords-candidates "1.5.1")
(defalias 'helm-c-bookmark-jump 'helm-bookmark-jump)
(make-obsolete 'helm-c-bookmark-jump 'helm-bookmark-jump "1.5.1")
(defalias 'helm-c-shadow-boring-buffers 'helm-shadow-boring-buffers)
(make-obsolete 'helm-c-shadow-boring-buffers 'helm-shadow-boring-buffers "1.5.1")
(defalias 'helm-c-buffer-match-major-mode 'helm-buffer-match-major-mode)
(make-obsolete 'helm-c-buffer-match-major-mode 'helm-buffer-match-major-mode "1.5.1")
(defalias 'helm-c-highlight-bookmark-su 'helm-highlight-bookmark-su)
(make-obsolete 'helm-c-highlight-bookmark-su 'helm-highlight-bookmark-su "1.5.1")
(defalias 'helm-c-grep-other-frame 'helm-grep-other-frame)
(make-obsolete 'helm-c-grep-other-frame 'helm-grep-other-frame "1.5.1")
(defalias 'helm-c-jabber-online-contacts 'helm-jabber-online-contacts)
(make-obsolete 'helm-c-jabber-online-contacts 'helm-jabber-online-contacts "1.5.1")
(defalias 'helm-c-apt-purge 'helm-apt-purge)
(make-obsolete 'helm-c-apt-purge 'helm-apt-purge "1.5.1")
(defalias 'helm-c-adaptive-sort 'helm-adaptive-sort)
(make-obsolete 'helm-c-adaptive-sort 'helm-adaptive-sort "1.5.1")
(defalias 'helm-c-bmkext-addressbook-setup-alist 'helm-bmkext-addressbook-setup-alist)
(make-obsolete 'helm-c-bmkext-addressbook-setup-alist 'helm-bmkext-addressbook-setup-alist "1.5.1")
(defalias 'helm-c-bbdb-collect-mail-addresses 'helm-bbdb-collect-mail-addresses)
(make-obsolete 'helm-c-bbdb-collect-mail-addresses 'helm-bbdb-collect-mail-addresses "1.5.1")
(defalias 'helm-c-bmkext-filter-setup-alist 'helm-bmkext-filter-setup-alist)
(make-obsolete 'helm-c-bmkext-filter-setup-alist 'helm-bmkext-filter-setup-alist "1.5.1")
(defalias 'helm-c-grep-buffers 'helm-grep-buffers)
(make-obsolete 'helm-c-grep-buffers 'helm-grep-buffers "1.5.1")
(defalias 'helm-c-apt-init 'helm-apt-init)
(make-obsolete 'helm-c-apt-init 'helm-apt-init "1.5.1")
(defalias 'helm-c-grep-run-save-buffer 'helm-grep-run-save-buffer)
(make-obsolete 'helm-c-grep-run-save-buffer 'helm-grep-run-save-buffer "1.5.1")
(defalias 'helm-c-simple-call-tree-init-base 'helm-simple-call-tree-init-base)
(make-obsolete 'helm-c-simple-call-tree-init-base 'helm-simple-call-tree-init-base "1.5.1")
(defalias 'helm-c-ggs-set-number-result 'helm-ggs-set-number-result)
(make-obsolete 'helm-c-ggs-set-number-result 'helm-ggs-set-number-result "1.5.1")
(defalias 'helm-c-grep-prepare-candidates 'helm-grep-prepare-candidates)
(make-obsolete 'helm-c-grep-prepare-candidates 'helm-grep-prepare-candidates "1.5.1")
(defalias 'helm-c-grep-jump-elscreen 'helm-grep-jump-elscreen)
(make-obsolete 'helm-c-grep-jump-elscreen 'helm-grep-jump-elscreen "1.5.1")
(defalias 'helm-c-show-completion-display-function 'helm-show-completion-display-function)
(make-obsolete 'helm-c-show-completion-display-function 'helm-show-completion-display-function "1.5.1")
(defalias 'helm-c-find-variable 'helm-find-variable)
(make-obsolete 'helm-c-find-variable 'helm-find-variable "1.5.1")
(defalias 'helm-c-build-elvi-list 'helm-build-elvi-list)
(make-obsolete 'helm-c-build-elvi-list 'helm-build-elvi-list "1.5.1")
(defalias 'helm-c-advice-toggle 'helm-advice-toggle)
(make-obsolete 'helm-c-advice-toggle 'helm-advice-toggle "1.5.1")
(defalias 'helm-c-buffers-persistent-kill 'helm-buffers-persistent-kill)
(make-obsolete 'helm-c-buffers-persistent-kill 'helm-buffers-persistent-kill "1.5.1")
(defalias 'helm-c-simple-call-tree-analyze-maybe 'helm-simple-call-tree-analyze-maybe)
(make-obsolete 'helm-c-simple-call-tree-analyze-maybe 'helm-simple-call-tree-analyze-maybe "1.5.1")
(defalias 'helm-c-register-action-transformer 'helm-register-action-transformer)
(make-obsolete 'helm-c-register-action-transformer 'helm-register-action-transformer "1.5.1")
(defalias 'helm-c-kill-ring-candidates 'helm-kill-ring-candidates)
(make-obsolete 'helm-c-kill-ring-candidates 'helm-kill-ring-candidates "1.5.1")
(defalias 'helm-c-bbdb-view-person-action 'helm-bbdb-view-person-action)
(make-obsolete 'helm-c-bbdb-view-person-action 'helm-bbdb-view-person-action "1.5.1")
(defalias 'helm-c-generic-browser 'helm-generic-browser)
(make-obsolete 'helm-c-generic-browser 'helm-generic-browser "1.5.1")
(defalias 'helm-c-top-init 'helm-top-init)
(make-obsolete 'helm-c-top-init 'helm-top-init "1.5.1")
(defalias 'helm-c-ucs-delete-backward 'helm-ucs-delete-backward)
(make-obsolete 'helm-c-ucs-delete-backward 'helm-ucs-delete-backward "1.5.1")
(defalias 'helm-c-firefox-bookmarks-get-value 'helm-firefox-bookmarks-get-value)
(make-obsolete 'helm-c-firefox-bookmarks-get-value 'helm-firefox-bookmarks-get-value "1.5.1")
(defalias 'helm-c-apt-generic-action 'helm-apt-generic-action)
(make-obsolete 'helm-c-apt-generic-action 'helm-apt-generic-action "1.5.1")
(defalias 'helm-c-pdfgrep-action-1 'helm-pdfgrep-action-1)
(make-obsolete 'helm-c-pdfgrep-action-1 'helm-pdfgrep-action-1 "1.5.1")
(defalias 'helm-c-buffer-query-replace-regexp 'helm-buffer-query-replace-regexp)
(make-obsolete 'helm-c-buffer-query-replace-regexp 'helm-buffer-query-replace-regexp "1.5.1")
(defalias 'helm-c-apropos-init 'helm-apropos-init)
(make-obsolete 'helm-c-apropos-init 'helm-apropos-init "1.5.1")
(defalias 'helm-c-emms-play-current-playlist 'helm-emms-play-current-playlist)
(make-obsolete 'helm-c-emms-play-current-playlist 'helm-emms-play-current-playlist "1.5.1")
(defalias 'helm-c-kill-new 'helm-kill-new)
(make-obsolete 'helm-c-kill-new 'helm-kill-new "1.5.1")
(defalias 'helm-c-bookmark-gnus-setup-alist 'helm-bookmark-gnus-setup-alist)
(make-obsolete 'helm-c-bookmark-gnus-setup-alist 'helm-bookmark-gnus-setup-alist "1.5.1")
(defalias 'helm-c-quit-and-execute-action 'helm-quit-and-execute-action)
(make-obsolete 'helm-c-quit-and-execute-action 'helm-quit-and-execute-action "1.5.1")
(defalias 'helm-c-build-info-index-command 'helm-build-info-index-command)
(make-obsolete 'helm-c-build-info-index-command 'helm-build-info-index-command "1.5.1")
(defalias 'helm-c-gentoo-setup-cache 'helm-gentoo-setup-cache)
(make-obsolete 'helm-c-gentoo-setup-cache 'helm-gentoo-setup-cache "1.5.1")
(defalias 'helm-c-ratpoison-commands-init 'helm-ratpoison-commands-init)
(make-obsolete 'helm-c-ratpoison-commands-init 'helm-ratpoison-commands-init "1.5.1")
(defalias 'helm-c-etags-select 'helm-etags-select)
(make-obsolete 'helm-c-etags-select 'helm-etags-select "1.5.1")
(defalias 'helm-c-etags-create-buffer 'helm-etags-create-buffer)
(make-obsolete 'helm-c-etags-create-buffer 'helm-etags-create-buffer "1.5.1")
(defalias 'helm-c-yahoo-suggest-set-candidates 'helm-yahoo-suggest-set-candidates)
(make-obsolete 'helm-c-yahoo-suggest-set-candidates 'helm-yahoo-suggest-set-candidates "1.5.1")
(defalias 'helm-c-org-keywords-show-help 'helm-org-keywords-show-help)
(make-obsolete 'helm-c-org-keywords-show-help 'helm-org-keywords-show-help "1.5.1")
(defalias 'helm-c-highlight-files 'helm-highlight-files)
(make-obsolete 'helm-c-highlight-files 'helm-highlight-files "1.5.1")
(defalias 'helm-c-get-pid-from-process-name 'helm-get-pid-from-process-name)
(make-obsolete 'helm-c-get-pid-from-process-name 'helm-get-pid-from-process-name "1.5.1")
(defalias 'helm-c-insert-latex-math 'helm-insert-latex-math)
(make-obsolete 'helm-c-insert-latex-math 'helm-insert-latex-math "1.5.1")
(defalias 'helm-c-buffer-query-replace 'helm-buffer-query-replace)
(make-obsolete 'helm-c-buffer-query-replace 'helm-buffer-query-replace "1.5.1")
(defalias 'helm-c-bmkext-run-edit 'helm-bmkext-run-edit)
(make-obsolete 'helm-c-bmkext-run-edit 'helm-bmkext-run-edit "1.5.1")
(defalias 'helm-c-skip-boring-files 'helm-skip-boring-files)
(make-obsolete 'helm-c-skip-boring-files 'helm-skip-boring-files "1.5.1")
(defalias 'helm-c-transform-file-load-el 'helm-transform-file-load-el)
(make-obsolete 'helm-c-transform-file-load-el 'helm-transform-file-load-el "1.5.1")
(defalias 'helm-c-ucs-insert-char 'helm-ucs-insert-char)
(make-obsolete 'helm-c-ucs-insert-char 'helm-ucs-insert-char "1.5.1")
(defalias 'helm-c-gentoo-setup-use-flags-cache 'helm-gentoo-setup-use-flags-cache)
(make-obsolete 'helm-c-gentoo-setup-use-flags-cache 'helm-gentoo-setup-use-flags-cache "1.5.1")
(defalias 'helm-c-buffer-help 'helm-buffer-help)
(make-obsolete 'helm-c-buffer-help 'helm-buffer-help "1.5.1")
(defalias 'helm-c-current-directory 'helm-current-directory)
(make-obsolete 'helm-c-current-directory 'helm-current-directory "1.5.1")
(defalias 'helm-c-highlight-firefox-bookmarks 'helm-highlight-firefox-bookmarks)
(make-obsolete 'helm-c-highlight-firefox-bookmarks 'helm-highlight-firefox-bookmarks "1.5.1")
(defalias 'helm-c-find-file-or-marked 'helm-find-file-or-marked)
(make-obsolete 'helm-c-find-file-or-marked 'helm-find-file-or-marked "1.5.1")
(defalias 'helm-c-query-replace-regexp 'helm-query-replace-regexp)
(make-obsolete 'helm-c-query-replace-regexp 'helm-query-replace-regexp "1.5.1")
(defalias 'helm-c-describe-variable 'helm-describe-variable)
(make-obsolete 'helm-c-describe-variable 'helm-describe-variable "1.5.1")
(defalias 'helm-c-advice-persistent-action 'helm-advice-persistent-action)
(make-obsolete 'helm-c-advice-persistent-action 'helm-advice-persistent-action "1.5.1")
(defalias 'helm-c-show-completion 'helm-show-completion)
(make-obsolete 'helm-c-show-completion 'helm-show-completion "1.5.1")
(defalias 'helm-c-google-suggest-set-candidates 'helm-google-suggest-set-candidates)
(make-obsolete 'helm-c-google-suggest-set-candidates 'helm-google-suggest-set-candidates "1.5.1")
(defalias 'helm-c-grep-persistent-action 'helm-grep-persistent-action)
(make-obsolete 'helm-c-grep-persistent-action 'helm-grep-persistent-action "1.5.1")
(defalias 'helm-c-open-file-with-default-tool 'helm-open-file-with-default-tool)
(make-obsolete 'helm-c-open-file-with-default-tool 'helm-open-file-with-default-tool "1.5.1")
(defalias 'helm-c-run-external-command 'helm-run-external-command)
(make-obsolete 'helm-c-run-external-command 'helm-run-external-command "1.5.1")
(defalias 'helm-c-bookmark-images-setup-alist 'helm-bookmark-images-setup-alist)
(make-obsolete 'helm-c-bookmark-images-setup-alist 'helm-bookmark-images-setup-alist "1.5.1")
(defalias 'helm-c-apt-candidate-transformer 'helm-apt-candidate-transformer)
(make-obsolete 'helm-c-apt-candidate-transformer 'helm-apt-candidate-transformer "1.5.1")
(defalias 'helm-c-grep-run-persistent-action 'helm-grep-run-persistent-action)
(make-obsolete 'helm-c-grep-run-persistent-action 'helm-grep-run-persistent-action "1.5.1")
(defalias 'helm-c-highlight-buffers 'helm-highlight-buffers)
(make-obsolete 'helm-c-highlight-buffers 'helm-highlight-buffers "1.5.1")
(defalias 'helm-c-man-default-action 'helm-man-default-action)
(make-obsolete 'helm-c-man-default-action 'helm-man-default-action "1.5.1")
(defalias 'helm-c-advice-candidates 'helm-advice-candidates)
(make-obsolete 'helm-c-advice-candidates 'helm-advice-candidates "1.5.1")
(defalias 'helm-c-stumpwm-commands-execute 'helm-stumpwm-commands-execute)
(make-obsolete 'helm-c-stumpwm-commands-execute 'helm-stumpwm-commands-execute "1.5.1")
(defalias 'helm-c-ucs-help 'helm-ucs-help)
(make-obsolete 'helm-c-ucs-help 'helm-ucs-help "1.5.1")
(defalias 'helm-c-apt-uninstall 'helm-apt-uninstall)
(make-obsolete 'helm-c-apt-uninstall 'helm-apt-uninstall "1.5.1")
(defalias 'helm-c-bookmark-run-jump-other-window 'helm-bookmark-run-jump-other-window)
(make-obsolete 'helm-c-bookmark-run-jump-other-window 'helm-bookmark-run-jump-other-window "1.5.1")
(defalias 'helm-c-elisp-library-scan-list 'helm-elisp-library-scan-list)
(make-obsolete 'helm-c-elisp-library-scan-list 'helm-elisp-library-scan-list "1.5.1")
(defalias 'helm-c-switch-to-buffer 'helm-switch-to-buffer)
(make-obsolete 'helm-c-switch-to-buffer 'helm-switch-to-buffer "1.5.1")
(defalias 'helm-c-bookmark-man-setup-alist 'helm-bookmark-man-setup-alist)
(make-obsolete 'helm-c-bookmark-man-setup-alist 'helm-bookmark-man-setup-alist "1.5.1")
(defalias 'helm-c-etags-find-tag-file-directory 'helm-etags-find-tag-file-directory)
(make-obsolete 'helm-c-etags-find-tag-file-directory 'helm-etags-find-tag-file-directory "1.5.1")
(defalias 'helm-c-xrandr-info 'helm-xrandr-info)
(make-obsolete 'helm-c-xrandr-info 'helm-xrandr-info "1.5.1")
(defalias 'helm-c-stringify 'helm-stringify)
(make-obsolete 'helm-c-stringify 'helm-stringify "1.5.1")
(defalias 'helm-c-goto-precedent-file 'helm-goto-precedent-file)
(make-obsolete 'helm-c-goto-precedent-file 'helm-goto-precedent-file "1.5.1")
(defalias 'helm-c-apt-cache-show 'helm-apt-cache-show)
(make-obsolete 'helm-c-apt-cache-show 'helm-apt-cache-show "1.5.1")
(defalias 'helm-c-etags-file-modified-p 'helm-etags-file-modified-p)
(make-obsolete 'helm-c-etags-file-modified-p 'helm-etags-file-modified-p "1.5.1")
(defalias 'helm-c-display-to-real-numbered-line 'helm-display-to-real-numbered-line)
(make-obsolete 'helm-c-display-to-real-numbered-line 'helm-display-to-real-numbered-line "1.5.1")
(defalias 'helm-c-etags-get-tag-file 'helm-etags-get-tag-file)
(make-obsolete 'helm-c-etags-get-tag-file 'helm-etags-get-tag-file "1.5.1")
(defalias 'helm-c-highlight-not-logged 'helm-highlight-not-logged)
(make-obsolete 'helm-c-highlight-not-logged 'helm-highlight-not-logged "1.5.1")
(defalias 'helm-c-pdfgrep-action 'helm-pdfgrep-action)
(make-obsolete 'helm-c-pdfgrep-action 'helm-pdfgrep-action "1.5.1")
(defalias 'helm-c-adaptive-save-history 'helm-adaptive-save-history)
(make-obsolete 'helm-c-adaptive-save-history 'helm-adaptive-save-history "1.5.1")
(defalias 'helm-c-get-first-line-documentation 'helm-get-first-line-documentation)
(make-obsolete 'helm-c-get-first-line-documentation 'helm-get-first-line-documentation "1.5.1")
(defalias 'helm-approximate-candidate-number 'helm-get-candidate-number)
(make-obsolete 'helm-approximate-candidate-number 'helm-get-candidate-number "1.5.5")


;;; variables
;;
;;
(defvaralias 'helm-c-bbdb-name 'helm-bbdb-name)
(make-obsolete-variable 'helm-c-bbdb-name 'helm-bbdb-name "1.5.1")
(defvaralias 'helm-c-pdfgrep-default-command 'helm-pdfgrep-default-command)
(make-obsolete-variable 'helm-c-pdfgrep-default-command 'helm-pdfgrep-default-command "1.5.1")
(defvaralias 'helm-c-source-time-world 'helm-source-time-world)
(make-obsolete-variable 'helm-c-source-time-world 'helm-source-time-world "1.5.1")
(defvaralias 'helm-c-grep-default-command 'helm-grep-default-command)
(make-obsolete-variable 'helm-c-grep-default-command 'helm-grep-default-command "1.5.1")
(defvaralias 'helm-c-default-info-index-list 'helm-default-info-index-list)
(make-obsolete-variable 'helm-c-default-info-index-list 'helm-default-info-index-list "1.5.1")
(defvaralias 'helm-c-etags-tag-file-dir 'helm-etags-tag-file-dir)
(make-obsolete-variable 'helm-c-etags-tag-file-dir 'helm-etags-tag-file-dir "1.5.1")
(defvaralias 'helm-c-source-man-pages 'helm-source-man-pages)
(make-obsolete-variable 'helm-c-source-man-pages 'helm-source-man-pages "1.5.1")
(defvaralias 'helm-c-source-bookmark-info 'helm-source-bookmark-info)
(make-obsolete-variable 'helm-c-source-bookmark-info 'helm-source-bookmark-info "1.5.1")
(defvaralias 'helm-c-file-cache-files 'helm-file-cache-files)
(make-obsolete-variable 'helm-c-file-cache-files 'helm-file-cache-files "1.5.1")
(defvaralias 'helm-c-ucs-map 'helm-ucs-map)
(make-obsolete-variable 'helm-c-ucs-map 'helm-ucs-map "1.5.1")
(defvaralias 'helm-c-show-completion-min-window-height 'helm-show-completion-min-window-height)
(make-obsolete-variable 'helm-c-show-completion-min-window-height 'helm-show-completion-min-window-height "1.5.1")
(defvaralias 'helm-c-adaptive-history 'helm-adaptive-history)
(make-obsolete-variable 'helm-c-adaptive-history 'helm-adaptive-history "1.5.1")
(defvaralias 'helm-c-source-bookmarks-local 'helm-source-bookmarks-local)
(make-obsolete-variable 'helm-c-source-bookmarks-local 'helm-source-bookmarks-local "1.5.1")
(defvaralias 'helm-c-grep-map 'helm-grep-map)
(make-obsolete-variable 'helm-c-grep-map 'helm-grep-map "1.5.1")
(defvaralias 'helm-c-source-session 'helm-source-session)
(make-obsolete-variable 'helm-c-source-session 'helm-source-session "1.5.1")
(defvaralias 'helm-c-source-org-headline 'helm-source-org-headline)
(make-obsolete-variable 'helm-c-source-org-headline 'helm-source-org-headline "1.5.1")
(defvaralias 'helm-c-source-minibuffer-history 'helm-source-minibuffer-history)
(make-obsolete-variable 'helm-c-source-minibuffer-history 'helm-source-minibuffer-history "1.5.1")
(defvaralias 'helm-c-source-files-in-current-dir 'helm-source-files-in-current-dir)
(make-obsolete-variable 'helm-c-source-files-in-current-dir 'helm-source-files-in-current-dir "1.5.1")
(defvaralias 'helm-c-source-etags-select 'helm-source-etags-select)
(make-obsolete-variable 'helm-c-source-etags-select 'helm-source-etags-select "1.5.1")
(defvaralias 'helm-c-gentoo-use-flags 'helm-gentoo-use-flags)
(make-obsolete-variable 'helm-c-gentoo-use-flags 'helm-gentoo-use-flags "1.5.1")
(defvaralias 'helm-c-show-completion-overlay 'helm-show-completion-overlay)
(make-obsolete-variable 'helm-c-show-completion-overlay 'helm-show-completion-overlay "1.5.1")
(defvaralias 'helm-c-buffer-help-message 'helm-buffer-help-message)
(make-obsolete-variable 'helm-c-buffer-help-message 'helm-buffer-help-message "1.5.1")
(defvaralias 'helm-c-ucs-max-len 'helm-ucs-max-len)
(make-obsolete-variable 'helm-c-ucs-max-len 'helm-ucs-max-len "1.5.1")
(defvaralias 'helm-c-imenu-index-filter 'helm-imenu-index-filter)
(make-obsolete-variable 'helm-c-imenu-index-filter 'helm-imenu-index-filter "1.5.1")
(defvaralias 'helm-c-org-keywords-info-location 'helm-org-keywords-info-location)
(make-obsolete-variable 'helm-c-org-keywords-info-location 'helm-org-keywords-info-location "1.5.1")
(defvaralias 'helm-c-kill-ring-max-lines-number 'helm-kill-ring-max-lines-number)
(make-obsolete-variable 'helm-c-kill-ring-max-lines-number 'helm-kill-ring-max-lines-number "1.5.1")
(defvaralias 'helm-c-source-apt 'helm-source-apt)
(make-obsolete-variable 'helm-c-source-apt 'helm-source-apt "1.5.1")
(defvaralias 'helm-c-bookmark-map 'helm-bookmark-map)
(make-obsolete-variable 'helm-c-bookmark-map 'helm-bookmark-map "1.5.1")
(defvaralias 'helm-c-source-xfonts 'helm-source-xfonts)
(make-obsolete-variable 'helm-c-source-xfonts 'helm-source-xfonts "1.5.1")
(defvaralias 'helm-c-source-kill-ring 'helm-source-kill-ring)
(make-obsolete-variable 'helm-c-source-kill-ring 'helm-source-kill-ring "1.5.1")
(defvaralias 'helm-c-home-url 'helm-home-url)
(make-obsolete-variable 'helm-c-home-url 'helm-home-url "1.5.1")
(defvaralias 'helm-c-etags-cache 'helm-etags-cache)
(make-obsolete-variable 'helm-c-etags-cache 'helm-etags-cache "1.5.1")
(defvaralias 'helm-c-source-latex-math 'helm-source-latex-math)
(make-obsolete-variable 'helm-c-source-latex-math 'helm-source-latex-math "1.5.1")
(defvaralias 'helm-c-source-bookmark-files&dirs 'helm-source-bookmark-files&dirs)
(make-obsolete-variable 'helm-c-source-bookmark-files&dirs 'helm-source-bookmark-files&dirs "1.5.1")
(defvaralias 'helm-c-source-emms-dired 'helm-source-emms-dired)
(make-obsolete-variable 'helm-c-source-emms-dired 'helm-source-emms-dired "1.5.1")
(defvaralias 'helm-c-source-firefox-bookmarks 'helm-source-firefox-bookmarks)
(make-obsolete-variable 'helm-c-source-firefox-bookmarks 'helm-source-firefox-bookmarks "1.5.1")
(defvaralias 'helm-c-source-ratpoison-commands 'helm-source-ratpoison-commands)
(make-obsolete-variable 'helm-c-source-ratpoison-commands 'helm-source-ratpoison-commands "1.5.1")
(defvaralias 'helm-c-apt-show-command 'helm-apt-show-command)
(make-obsolete-variable 'helm-c-apt-show-command 'helm-apt-show-command "1.5.1")
(defvaralias 'helm-c-pdfgrep-default-function 'helm-pdfgrep-default-function)
(make-obsolete-variable 'helm-c-pdfgrep-default-function 'helm-pdfgrep-default-function "1.5.1")
(defvaralias 'helm-c-simple-call-tree-tick 'helm-simple-call-tree-tick)
(make-obsolete-variable 'helm-c-simple-call-tree-tick 'helm-simple-call-tree-tick "1.5.1")
(defvaralias 'helm-c-source-register 'helm-source-register)
(make-obsolete-variable 'helm-c-source-register 'helm-source-register "1.5.1")
(defvaralias 'helm-c-boring-buffer-regexp-list 'helm-boring-buffer-regexp-list)
(make-obsolete-variable 'helm-c-boring-buffer-regexp-list 'helm-boring-buffer-regexp-list "1.5.1")
(defvaralias 'helm-c-source-locate 'helm-source-locate)
(make-obsolete-variable 'helm-c-source-locate 'helm-source-locate "1.5.1")
(defvaralias 'helm-c-source-advice 'helm-source-advice)
(make-obsolete-variable 'helm-c-source-advice 'helm-source-advice "1.5.1")
(defvaralias 'helm-c-source-yaoddmuse-emacswiki-edit-or-view 'helm-source-yaoddmuse-emacswiki-edit-or-view)
(make-obsolete-variable 'helm-c-source-yaoddmuse-emacswiki-edit-or-view 'helm-source-yaoddmuse-emacswiki-edit-or-view "1.5.1")
(defvaralias 'helm-c-ucs-mode-line-string 'helm-ucs-mode-line-string)
(make-obsolete-variable 'helm-c-ucs-mode-line-string 'helm-ucs-mode-line-string "1.5.1")
(defvaralias 'helm-c-xfonts-cache 'helm-xfonts-cache)
(make-obsolete-variable 'helm-c-xfonts-cache 'helm-xfonts-cache "1.5.1")
(defvaralias 'helm-c-source-yahoo-suggest 'helm-source-yahoo-suggest)
(make-obsolete-variable 'helm-c-source-yahoo-suggest 'helm-source-yahoo-suggest "1.5.1")
(defvaralias 'helm-c-rzgrep-cache 'helm-rzgrep-cache)
(make-obsolete-variable 'helm-c-rzgrep-cache 'helm-rzgrep-cache "1.5.1")
(defvaralias 'helm-c-source-org-keywords 'helm-source-org-keywords)
(make-obsolete-variable 'helm-c-source-org-keywords 'helm-source-org-keywords "1.5.1")
(defvaralias 'helm-c-pdfgrep-map 'helm-pdfgrep-map)
(make-obsolete-variable 'helm-c-pdfgrep-map 'helm-pdfgrep-map "1.5.1")
(defvaralias 'helm-c-source-lacarte 'helm-source-lacarte)
(make-obsolete-variable 'helm-c-source-lacarte 'helm-source-lacarte "1.5.1")
(defvaralias 'helm-c-apt-search-command 'helm-apt-search-command)
(make-obsolete-variable 'helm-c-apt-search-command 'helm-apt-search-command "1.5.1")
(defvaralias 'helm-c-default-external-file-browser 'helm-default-external-file-browser)
(make-obsolete-variable 'helm-c-default-external-file-browser 'helm-default-external-file-browser "1.5.1")
(defvaralias 'helm-c-source-evaluation-result 'helm-source-evaluation-result)
(make-obsolete-variable 'helm-c-source-evaluation-result 'helm-source-evaluation-result "1.5.1")
(defvaralias 'helm-c-yahoo-suggest-search-url 'helm-yahoo-suggest-search-url)
(make-obsolete-variable 'helm-c-yahoo-suggest-search-url 'helm-yahoo-suggest-search-url "1.5.1")
(defvaralias 'helm-c-source-occur 'helm-source-occur)
(make-obsolete-variable 'helm-c-source-occur 'helm-source-occur "1.5.1")
(defvaralias 'helm-c-source-calculation-result 'helm-source-calculation-result)
(make-obsolete-variable 'helm-c-source-calculation-result 'helm-source-calculation-result "1.5.1")
(defvaralias 'helm-c-default-zgrep-command 'helm-default-zgrep-command)
(make-obsolete-variable 'helm-c-default-zgrep-command 'helm-default-zgrep-command "1.5.1")
(defvaralias 'helm-c-show-info-in-mode-line-delay 'helm-show-info-in-mode-line-delay)
(make-obsolete-variable 'helm-c-show-info-in-mode-line-delay 'helm-show-info-in-mode-line-delay "1.5.1")
(defvaralias 'helm-c-source-ido-virtual-buffers 'helm-source-ido-virtual-buffers)
(make-obsolete-variable 'helm-c-source-ido-virtual-buffers 'helm-source-ido-virtual-buffers "1.5.1")
(defvaralias 'helm-c-source-yaoddmuse-emacswiki-post-library 'helm-source-yaoddmuse-emacswiki-post-library)
(make-obsolete-variable 'helm-c-source-yaoddmuse-emacswiki-post-library 'helm-source-yaoddmuse-emacswiki-post-library "1.5.1")
(defvaralias 'helm-c-source-top 'helm-source-top)
(make-obsolete-variable 'helm-c-source-top 'helm-source-top "1.5.1")
(defvaralias 'helm-c-source-colors 'helm-source-colors)
(make-obsolete-variable 'helm-c-source-colors 'helm-source-colors "1.5.1")
(defvaralias 'helm-c-read-file-map 'helm-read-file-map)
(make-obsolete-variable 'helm-c-read-file-map 'helm-read-file-map "1.5.1")
(defvaralias 'helm-c-source-file-cache 'helm-source-file-cache)
(make-obsolete-variable 'helm-c-source-file-cache 'helm-source-file-cache "1.5.1")
(defvaralias 'helm-c-google-suggest-default-browser-function 'helm-google-suggest-default-browser-function)
(make-obsolete-variable 'helm-c-google-suggest-default-browser-function 'helm-google-suggest-default-browser-function "1.5.1")
(defvaralias 'helm-c-external-programs-associations 'helm-external-programs-associations)
(make-obsolete-variable 'helm-c-external-programs-associations 'helm-external-programs-associations "1.5.1")
(defvaralias 'helm-c-source-complex-command-history 'helm-source-complex-command-history)
(make-obsolete-variable 'helm-c-source-complex-command-history 'helm-source-complex-command-history "1.5.1")
(defvaralias 'helm-c-cached-imenu-candidates 'helm-cached-imenu-candidates)
(make-obsolete-variable 'helm-c-cached-imenu-candidates 'helm-cached-imenu-candidates "1.5.1")
(defvaralias 'helm-c-gentoo-buffer 'helm-gentoo-buffer)
(make-obsolete-variable 'helm-c-gentoo-buffer 'helm-gentoo-buffer "1.5.1")
(defvaralias 'helm-c-source-simple-call-tree-callers-functions 'helm-source-simple-call-tree-callers-functions)
(make-obsolete-variable 'helm-c-source-simple-call-tree-callers-functions 'helm-source-simple-call-tree-callers-functions "1.5.1")
(defvaralias 'helm-c-external-commands-list 'helm-external-commands-list)
(make-obsolete-variable 'helm-c-external-commands-list 'helm-external-commands-list "1.5.1")
(defvaralias 'helm-c-source-ucs 'helm-source-ucs)
(make-obsolete-variable 'helm-c-source-ucs 'helm-source-ucs "1.5.1")
(defvaralias 'helm-c-adaptive-history-length 'helm-adaptive-history-length)
(make-obsolete-variable 'helm-c-adaptive-history-length 'helm-adaptive-history-length "1.5.1")
(defvaralias 'helm-c-source-customize-face 'helm-source-customize-face)
(make-obsolete-variable 'helm-c-source-customize-face 'helm-source-customize-face "1.5.1")
(defvaralias 'helm-c-source-bookmarks-ssh 'helm-source-bookmarks-ssh)
(make-obsolete-variable 'helm-c-source-bookmarks-ssh 'helm-source-bookmarks-ssh "1.5.1")
(defvaralias 'helm-c-eldoc-in-minibuffer-show-fn 'helm-eldoc-in-minibuffer-show-fn)
(make-obsolete-variable 'helm-c-eldoc-in-minibuffer-show-fn 'helm-eldoc-in-minibuffer-show-fn "1.5.1")
(defvaralias 'helm-c-google-suggest-search-url 'helm-google-suggest-search-url)
(make-obsolete-variable 'helm-c-google-suggest-search-url 'helm-google-suggest-search-url "1.5.1")
(defvaralias 'helm-c-source-bmkext-addressbook 'helm-source-bmkext-addressbook)
(make-obsolete-variable 'helm-c-source-bmkext-addressbook 'helm-source-bmkext-addressbook "1.5.1")
(defvaralias 'helm-c-source-ctags 'helm-source-ctags)
(make-obsolete-variable 'helm-c-source-ctags 'helm-source-ctags "1.5.1")
(defvaralias 'helm-c-source-eev-anchor 'helm-source-eev-anchor)
(make-obsolete-variable 'helm-c-source-eev-anchor 'helm-source-eev-anchor "1.5.1")
(defvaralias 'helm-c-apt-installed-packages 'helm-apt-installed-packages)
(make-obsolete-variable 'helm-c-apt-installed-packages 'helm-apt-installed-packages "1.5.1")
(defvaralias 'helm-c-apt-all-packages 'helm-apt-all-packages)
(make-obsolete-variable 'helm-c-apt-all-packages 'helm-apt-all-packages "1.5.1")
(defvaralias 'helm-c-find-files-doc-header 'helm-find-files-doc-header)
(make-obsolete-variable 'helm-c-find-files-doc-header 'helm-find-files-doc-header "1.5.1")
(defvaralias 'helm-c-source-imenu 'helm-source-imenu)
(make-obsolete-variable 'helm-c-source-imenu 'helm-source-imenu "1.5.1")
(defvaralias 'helm-c-grep-default-function 'helm-grep-default-function)
(make-obsolete-variable 'helm-c-grep-default-function 'helm-grep-default-function "1.5.1")
(defvaralias 'helm-c-top-command 'helm-top-command)
(make-obsolete-variable 'helm-c-top-command 'helm-top-command "1.5.1")
(defvaralias 'helm-c-source-bookmark-man 'helm-source-bookmark-man)
(make-obsolete-variable 'helm-c-source-bookmark-man 'helm-source-bookmark-man "1.5.1")
(defvaralias 'helm-c-source-stumpwm-commands 'helm-source-stumpwm-commands)
(make-obsolete-variable 'helm-c-source-stumpwm-commands 'helm-source-stumpwm-commands "1.5.1")
(defvaralias 'helm-c-source-idle-time-timers 'helm-source-idle-time-timers)
(make-obsolete-variable 'helm-c-source-idle-time-timers 'helm-source-idle-time-timers "1.5.1")
(defvaralias 'helm-c-turn-on-show-completion 'helm-turn-on-show-completion)
(make-obsolete-variable 'helm-c-turn-on-show-completion 'helm-turn-on-show-completion "1.5.1")
(defvaralias 'helm-c-esh-help-message 'helm-esh-help-message)
(make-obsolete-variable 'helm-c-esh-help-message 'helm-esh-help-message "1.5.1")
(defvaralias 'helm-c-register-max-offset 'helm-register-max-offset)
(make-obsolete-variable 'helm-c-register-max-offset 'helm-register-max-offset "1.5.1")
(defvaralias 'helm-c-source-bookmark-set 'helm-source-bookmark-set)
(make-obsolete-variable 'helm-c-source-bookmark-set 'helm-source-bookmark-set "1.5.1")
(defvaralias 'helm-c-source-file-name-history 'helm-source-file-name-history)
(make-obsolete-variable 'helm-c-source-file-name-history 'helm-source-file-name-history "1.5.1")
(defvaralias 'helm-c-file-cache-initialized-p 'helm-file-cache-initialized-p)
(make-obsolete-variable 'helm-c-file-cache-initialized-p 'helm-file-cache-initialized-p "1.5.1")
(defvaralias 'helm-c-grep-max-length-history 'helm-grep-max-length-history)
(make-obsolete-variable 'helm-c-grep-max-length-history 'helm-grep-max-length-history "1.5.1")
(defvaralias 'helm-c-source-jabber-contacts 'helm-source-jabber-contacts)
(make-obsolete-variable 'helm-c-source-jabber-contacts 'helm-source-jabber-contacts "1.5.1")
(defvaralias 'helm-c-grep-use-ioccur-style-keys 'helm-grep-use-ioccur-style-keys)
(make-obsolete-variable 'helm-c-grep-use-ioccur-style-keys 'helm-grep-use-ioccur-style-keys "1.5.1")
(defvaralias 'helm-c-ctags-modes 'helm-ctags-modes)
(make-obsolete-variable 'helm-c-ctags-modes 'helm-ctags-modes "1.5.1")
(defvaralias 'helm-c-source-moccur 'helm-source-moccur)
(make-obsolete-variable 'helm-c-source-moccur 'helm-source-moccur "1.5.1")
(defvaralias 'helm-c-source-w3m-bookmarks 'helm-source-w3m-bookmarks)
(make-obsolete-variable 'helm-c-source-w3m-bookmarks 'helm-source-w3m-bookmarks "1.5.1")
(defvaralias 'helm-c-locate-command 'helm-locate-command)
(make-obsolete-variable 'helm-c-locate-command 'helm-locate-command "1.5.1")
(defvaralias 'helm-c-etags-tag-file-name 'helm-etags-tag-file-name)
(make-obsolete-variable 'helm-c-etags-tag-file-name 'helm-etags-tag-file-name "1.5.1")
(defvaralias 'helm-c-source-bookmarks-su 'helm-source-bookmarks-su)
(make-obsolete-variable 'helm-c-source-bookmarks-su 'helm-source-bookmarks-su "1.5.1")
(defvaralias 'helm-c-source-emms-streams 'helm-source-emms-streams)
(make-obsolete-variable 'helm-c-source-emms-streams 'helm-source-emms-streams "1.5.1")
(defvaralias 'helm-c-buffer-map 'helm-buffer-map)
(make-obsolete-variable 'helm-c-buffer-map 'helm-buffer-map "1.5.1")
(defvaralias 'helm-c-source-files-in-all-dired 'helm-source-files-in-all-dired)
(make-obsolete-variable 'helm-c-source-files-in-all-dired 'helm-source-files-in-all-dired "1.5.1")
(defvaralias 'helm-c-source-eshell-history 'helm-source-eshell-history)
(make-obsolete-variable 'helm-c-source-eshell-history 'helm-source-eshell-history "1.5.1")
(defvaralias 'helm-c-source-absolute-time-timers 'helm-source-absolute-time-timers)
(make-obsolete-variable 'helm-c-source-absolute-time-timers 'helm-source-absolute-time-timers "1.5.1")
(defvaralias 'helm-c-source-findutils 'helm-source-findutils)
(make-obsolete-variable 'helm-c-source-findutils 'helm-source-findutils "1.5.1")
(defvaralias 'helm-c-ucs-help-message 'helm-ucs-help-message)
(make-obsolete-variable 'helm-c-ucs-help-message 'helm-ucs-help-message "1.5.1")
(defvaralias 'helm-c-source-find-files 'helm-source-find-files)
(make-obsolete-variable 'helm-c-source-find-files 'helm-source-find-files "1.5.1")
(defvaralias 'helm-c-source-recentf 'helm-source-recentf)
(make-obsolete-variable 'helm-c-source-recentf 'helm-source-recentf "1.5.1")
(defvaralias 'helm-c-cache-world 'helm-cache-world)
(make-obsolete-variable 'helm-c-cache-world 'helm-cache-world "1.5.1")
(defvaralias 'helm-c-source-picklist 'helm-source-picklist)
(make-obsolete-variable 'helm-c-source-picklist 'helm-source-picklist "1.5.1")
(defvaralias 'helm-c-source-buffer-not-found 'helm-source-buffer-not-found)
(make-obsolete-variable 'helm-c-source-buffer-not-found 'helm-source-buffer-not-found "1.5.1")
(defvaralias 'helm-c-source-xrandr-change-resolution 'helm-source-xrandr-change-resolution)
(make-obsolete-variable 'helm-c-source-xrandr-change-resolution 'helm-source-xrandr-change-resolution "1.5.1")
(defvaralias 'helm-c-source-bookmark-w3m 'helm-source-bookmark-w3m)
(make-obsolete-variable 'helm-c-source-bookmark-w3m 'helm-source-bookmark-w3m "1.5.1")
(defvaralias 'helm-c-google-suggest-url 'helm-google-suggest-url)
(make-obsolete-variable 'helm-c-google-suggest-url 'helm-google-suggest-url "1.5.1")
(defvaralias 'helm-c-source-semantic 'helm-source-semantic)
(make-obsolete-variable 'helm-c-source-semantic 'helm-source-semantic "1.5.1")
(defvaralias 'helm-c-source-mark-ring 'helm-source-mark-ring)
(make-obsolete-variable 'helm-c-source-mark-ring 'helm-source-mark-ring "1.5.1")
(defvaralias 'helm-c-source-fixme 'helm-source-fixme)
(make-obsolete-variable 'helm-c-source-fixme 'helm-source-fixme "1.5.1")
(defvaralias 'helm-c-grep-preferred-ext 'helm-grep-preferred-ext)
(make-obsolete-variable 'helm-c-grep-preferred-ext 'helm-grep-preferred-ext "1.5.1")
(defvaralias 'helm-c-source-gentoo 'helm-source-gentoo)
(make-obsolete-variable 'helm-c-source-gentoo 'helm-source-gentoo "1.5.1")
(defvaralias 'helm-c-source-bookmarks 'helm-source-bookmarks)
(make-obsolete-variable 'helm-c-source-bookmarks 'helm-source-bookmarks "1.5.1")
(defvaralias 'helm-c-source-elisp-library-scan 'helm-source-elisp-library-scan)
(make-obsolete-variable 'helm-c-source-elisp-library-scan 'helm-source-elisp-library-scan "1.5.1")
(defvaralias 'helm-c-source-info-pages 'helm-source-info-pages)
(make-obsolete-variable 'helm-c-source-info-pages 'helm-source-info-pages "1.5.1")
(defvaralias 'helm-c-source-bookmark-gnus 'helm-source-bookmark-gnus)
(make-obsolete-variable 'helm-c-source-bookmark-gnus 'helm-source-bookmark-gnus "1.5.1")
(defvaralias 'helm-c-source-elscreen 'helm-source-elscreen)
(make-obsolete-variable 'helm-c-source-elscreen 'helm-source-elscreen "1.5.1")
(defvaralias 'helm-c-source-emacs-source-defun 'helm-source-emacs-source-defun)
(make-obsolete-variable 'helm-c-source-emacs-source-defun 'helm-source-emacs-source-defun "1.5.1")
(defvaralias 'helm-c-source-emacs-lisp-toplevels 'helm-source-emacs-lisp-toplevels)
(make-obsolete-variable 'helm-c-source-emacs-lisp-toplevels 'helm-source-emacs-lisp-toplevels "1.5.1")
(defvaralias 'helm-c-etags-mtime-alist 'helm-etags-mtime-alist)
(make-obsolete-variable 'helm-c-etags-mtime-alist 'helm-etags-mtime-alist "1.5.1")
(defvaralias 'helm-c-info-pages 'helm-info-pages)
(make-obsolete-variable 'helm-c-info-pages 'helm-info-pages "1.5.1")
(defvaralias 'helm-c-grep-default-recurse-command 'helm-grep-default-recurse-command)
(make-obsolete-variable 'helm-c-grep-default-recurse-command 'helm-grep-default-recurse-command "1.5.1")
(defvaralias 'helm-c-yaoddmuse-ew-cache 'helm-yaoddmuse-ew-cache)
(make-obsolete-variable 'helm-c-yaoddmuse-ew-cache 'helm-yaoddmuse-ew-cache "1.5.1")
(defvaralias 'helm-c-source-google-suggest 'helm-source-google-suggest)
(make-obsolete-variable 'helm-c-source-google-suggest 'helm-source-google-suggest "1.5.1")
(defvaralias 'helm-c-etags-tag-file-search-limit 'helm-etags-tag-file-search-limit)
(make-obsolete-variable 'helm-c-etags-tag-file-search-limit 'helm-etags-tag-file-search-limit "1.5.1")
(defvaralias 'helm-c-w3m-bookmarks-alist 'helm-w3m-bookmarks-alist)
(make-obsolete-variable 'helm-c-w3m-bookmarks-alist 'helm-w3m-bookmarks-alist "1.5.1")
(defvaralias 'helm-c-source-ff-file-name-history 'helm-source-ff-file-name-history)
(make-obsolete-variable 'helm-c-source-ff-file-name-history 'helm-source-ff-file-name-history "1.5.1")
(defvaralias 'helm-c-source-buffers-list 'helm-source-buffers-list)
(make-obsolete-variable 'helm-c-source-buffers-list 'helm-source-buffers-list "1.5.1")
(defvaralias 'helm-c-source-simple-call-tree-functions-callers 'helm-source-simple-call-tree-functions-callers)
(make-obsolete-variable 'helm-c-source-simple-call-tree-functions-callers 'helm-source-simple-call-tree-functions-callers "1.5.1")
(defvaralias 'helm-c-cached-imenu-alist 'helm-cached-imenu-alist)
(make-obsolete-variable 'helm-c-cached-imenu-alist 'helm-cached-imenu-alist "1.5.1")
(defvaralias 'helm-c-source-emacs-lisp-expectations 'helm-source-emacs-lisp-expectations)
(make-obsolete-variable 'helm-c-source-emacs-lisp-expectations 'helm-source-emacs-lisp-expectations "1.5.1")
(defvaralias 'helm-c-source-rd-headline 'helm-source-rd-headline)
(make-obsolete-variable 'helm-c-source-rd-headline 'helm-source-rd-headline "1.5.1")
(defvaralias 'helm-c-source-tracker-search 'helm-source-tracker-search)
(make-obsolete-variable 'helm-c-source-tracker-search 'helm-source-tracker-search "1.5.1")
(defvaralias 'helm-c-simple-call-tree-related-functions 'helm-simple-call-tree-related-functions)
(make-obsolete-variable 'helm-c-simple-call-tree-related-functions 'helm-simple-call-tree-related-functions "1.5.1")
(defvaralias 'helm-c-yaoddmuse-cache-file 'helm-yaoddmuse-cache-file)
(make-obsolete-variable 'helm-c-yaoddmuse-cache-file 'helm-yaoddmuse-cache-file "1.5.1")
(defvaralias 'helm-c-google-suggest-default-function 'helm-google-suggest-default-function)
(make-obsolete-variable 'helm-c-google-suggest-default-function 'helm-google-suggest-default-function "1.5.1")
(defvaralias 'helm-c-imenu-delimiter 'helm-imenu-delimiter)
(make-obsolete-variable 'helm-c-imenu-delimiter 'helm-imenu-delimiter "1.5.1")
(defvaralias 'helm-c-source-emacs-process 'helm-source-emacs-process)
(make-obsolete-variable 'helm-c-source-emacs-process 'helm-source-emacs-process "1.5.1")
(defvaralias 'helm-c-firefox-bookmarks-alist 'helm-firefox-bookmarks-alist)
(make-obsolete-variable 'helm-c-firefox-bookmarks-alist 'helm-firefox-bookmarks-alist "1.5.1")
(defvaralias 'helm-c-simple-call-tree-function-index 'helm-simple-call-tree-function-index)
(make-obsolete-variable 'helm-c-simple-call-tree-function-index 'helm-simple-call-tree-function-index "1.5.1")
(defvaralias 'helm-c-etags-map 'helm-etags-map)
(make-obsolete-variable 'helm-c-etags-map 'helm-etags-map "1.5.1")
(defvaralias 'helm-c-grep-history 'helm-grep-history)
(make-obsolete-variable 'helm-c-grep-history 'helm-grep-history "1.5.1")
(defvaralias 'helm-c-cached-imenu-tick 'helm-cached-imenu-tick)
(make-obsolete-variable 'helm-c-cached-imenu-tick 'helm-cached-imenu-tick "1.5.1")
(defvaralias 'helm-c-source-esh 'helm-source-esh)
(make-obsolete-variable 'helm-c-source-esh 'helm-source-esh "1.5.1")
(defvaralias 'helm-c-source-global-mark-ring 'helm-source-global-mark-ring)
(make-obsolete-variable 'helm-c-source-global-mark-ring 'helm-source-global-mark-ring "1.5.1")
(defvaralias 'helm-c-man-pages 'helm-man-pages)
(make-obsolete-variable 'helm-c-man-pages 'helm-man-pages "1.5.1")
(defvaralias 'helm-c-adaptive-done 'helm-adaptive-done)
(make-obsolete-variable 'helm-c-adaptive-done 'helm-adaptive-done "1.5.1")
(defvaralias 'helm-c-source-bbdb 'helm-source-bbdb)
(make-obsolete-variable 'helm-c-source-bbdb 'helm-source-bbdb "1.5.1")
(defvaralias 'helm-c-source-regexp 'helm-source-regexp)
(make-obsolete-variable 'helm-c-source-regexp 'helm-source-regexp "1.5.1")
(defvaralias 'helm-c-boring-file-regexp-list 'helm-boring-file-regexp-list)
(make-obsolete-variable 'helm-c-boring-file-regexp-list 'helm-boring-file-regexp-list "1.5.1")
(defvaralias 'helm-c-show-completion-use-special-display 'helm-show-completion-use-special-display)
(make-obsolete-variable 'helm-c-show-completion-use-special-display 'helm-show-completion-use-special-display "1.5.1")
(defvaralias 'helm-c-apt-query 'helm-apt-query)
(make-obsolete-variable 'helm-c-apt-query 'helm-apt-query "1.5.1")
(defvaralias 'helm-c-pdfgrep-default-read-command 'helm-pdfgrep-default-read-command)
(make-obsolete-variable 'helm-c-pdfgrep-default-read-command 'helm-pdfgrep-default-read-command "1.5.1")
(defvaralias 'helm-c-source-bookmark-images 'helm-source-bookmark-images)
(make-obsolete-variable 'helm-c-source-bookmark-images 'helm-source-bookmark-images "1.5.1")
(defvaralias 'helm-c-source-mac-spotlight 'helm-source-mac-spotlight)
(make-obsolete-variable 'helm-c-source-mac-spotlight 'helm-source-mac-spotlight "1.5.1")
(defvaralias 'helm-c-yahoo-suggest-url 'helm-yahoo-suggest-url)
(make-obsolete-variable 'helm-c-yahoo-suggest-url 'helm-yahoo-suggest-url "1.5.1")
(defvaralias 'helm-c-source-oddmuse-headline 'helm-source-oddmuse-headline)
(make-obsolete-variable 'helm-c-source-oddmuse-headline 'helm-source-oddmuse-headline "1.5.1")
(defvaralias 'helm-c-cache-gentoo 'helm-cache-gentoo)
(make-obsolete-variable 'helm-c-cache-gentoo 'helm-cache-gentoo "1.5.1")
(defvaralias 'helm-c-zgrep-recurse-flag 'helm-zgrep-recurse-flag)
(make-obsolete-variable 'helm-c-zgrep-recurse-flag 'helm-zgrep-recurse-flag "1.5.1")
(defvaralias 'helm-c-source-use-flags 'helm-source-use-flags)
(make-obsolete-variable 'helm-c-source-use-flags 'helm-source-use-flags "1.5.1")
(defvaralias 'helm-c-moccur-map 'helm-moccur-map)
(make-obsolete-variable 'helm-c-moccur-map 'helm-moccur-map "1.5.1")
(defvaralias 'helm-c-adaptive-history-file 'helm-adaptive-history-file)
(make-obsolete-variable 'helm-c-adaptive-history-file 'helm-adaptive-history-file "1.5.1")
(defvaralias 'helm-c-grep-default-directory-fn 'helm-grep-default-directory-fn)
(make-obsolete-variable 'helm-c-grep-default-directory-fn 'helm-grep-default-directory-fn "1.5.1")
(defvaralias 'helm-c-apt-input-history 'helm-apt-input-history)
(make-obsolete-variable 'helm-c-apt-input-history 'helm-apt-input-history "1.5.1")
(defvaralias 'helm-c-source-emms-files 'helm-source-emms-files)
(make-obsolete-variable 'helm-c-source-emms-files 'helm-source-emms-files "1.5.1")


(provide 'helm-aliases)

;;; helm-aliases.el ends here
